import { notification } from 'antd'
import { createSearchParams, history } from '@umijs/max'
import RightContent from '@/components/RightContent'
import consts from './utils/consts'
import { queryCurrentUser } from './services/api/login'
import defaultSettings from '../config/defaultSettings'
import type { Settings as LayoutSettings } from '@ant-design/pro-layout'
import type { RequestConfig, RunTimeLayoutConfig } from '@umijs/max'

export async function getInitialState(): Promise<{
  settings?: Partial<LayoutSettings>
  currentUser?: API.CurrentUser
  fetchUserInfo?: () => Promise<API.CurrentUser | undefined>
}> {
  const fetchUserInfo = async () => {
    try {
      const msg = await queryCurrentUser()
      return msg.data
    } catch (error) {
      history.push(consts.loginPath)
    }
    return undefined
  }
  // 如果是登录页面，不执行
  if (history.location.pathname !== consts.loginPath) {
    const currentUser = await fetchUserInfo()
    return {
      fetchUserInfo,
      ...currentUser,
      settings: {}
    }
  }
  return {
    fetchUserInfo,
    settings: {}
  }
}

const authHeaderInterceptor = options => {
  const token = window.localStorage.getItem('TOKEN_ID')
  // 如果是登录页面，不执行
  if (token) {
    // 在白名单里的请求放过
    if (consts.TOKEN_WHITE_LIST.includes(options.url)) {
      return options
    }
    options.headers[consts.TOKEN_HEADER] = `bearer ${token}`
  }
  return options
}

const errorHandler = (error: any, opts: any) => {
  if (opts?.skipErrorHandler) return

  const errorInfo = error.info
  if (errorInfo) {
    const { errorMessage = '请求失败', errorCode } = errorInfo
    if (consts.TOKEN_INVALID_CODE.includes(errorCode) && window.location.pathname !== consts.loginPath) {
      notification.error({
        message: '用户信息过期',
        description: '请重新登录'
      })
      history.replace({
        pathname: consts.loginPath,
        search: createSearchParams({
          redirect: window.location.pathname
        }).toString()
      })
      return
    }
    notification.error({
      message: '请求失败',
      description: errorMessage
    })
  } else if (error.response) {
    // Axios 的错误
    // 请求成功发出且服务器也响应了状态码，但状态代码超出了 2xx 的范围
    notification.error({
      description: `状态码为${error.status}, 请联系管理员进行处理`,
      message: '请求异常'
    })
  } else if (error.request) {
    // 请求已经成功发起，但没有收到响应
    // 或请求根本没有发送出去
    notification.error({
      description: '您的网络请求已经发起，但没有收到响应',
      message: '响应超时'
    })
  } else {
    notification.error({
      description: '您的网络发生异常，无法连接服务器',
      message: '网络异常'
    })
  }
}

const responseInterceptor = response => {
  const { data, code: errorCode, msg: errorMessage } = response.data
  if (!errorCode || errorCode !== consts.RET_CODE.SUCCESS) {
    if (errorCode !== consts.RET_CODE.SUCCESS) {
      const error: any = new Error(errorMessage)
      error.name = 'BizError'
      error.info = { errorCode, errorMessage, data }
      throw error
    }
  }
  return Promise.resolve(response)
}
export const request: RequestConfig = {
  errorConfig: {
    errorHandler
  },
  baseURL: consts.PREFIX_URL,
  responseInterceptors: [responseInterceptor],
  requestInterceptors: [authHeaderInterceptor]
}

export const layout: RunTimeLayoutConfig = ({ initialState }) => {
  return {
    rightContentRender: () => <RightContent />,
    disableContentMargin: false,
    onPageChange: () => {
      if (!initialState?.currentUser?.ID && window.location.pathname !== consts.loginPath) {
        history.replace({
          pathname: consts.loginPath,
          search: createSearchParams({
            redirect: history.location.pathname
          }).toString()
        })
      }
    },
    ...defaultSettings
  }
}
